# NOTE: This module has no unit tests.

import binstar_client
from conda_build.build import bldpkg_path
from .conda_interface import get_index, subdir


def distribution_exists(binstar_cli, owner, metadata):
    """
    Determine whether a distribution exists.

    This does not check specific channels - it is either on binstar or it is not.
    """
    fname = '{}/{}.tar.bz2'.format(subdir, metadata.dist())
    try:
        r = binstar_cli.distribution(owner, metadata.name(), metadata.version(),
                                     fname)
        exists = True
    except binstar_client.NotFound:
        exists = False
    return exists


def distribution_exists_on_channel(binstar_cli, owner, metadata, channel='main'):
    """
    Determine whether a distribution exists on a specific channel.

    Note from @pelson: As far as I can see, there is no easy way to do this on binstar.

    """
    fname = '{}.tar.bz2'.format(metadata.dist())
    channel_url = '/'.join([owner, 'label', channel])

    distributions_on_channel = get_index([channel_url],
                                         prepend=False, use_cache=False)

    try:
        on_channel = (distributions_on_channel[fname]['subdir'] ==
                      subdir)
    except KeyError:
        on_channel = False
    return on_channel


def add_distribution_to_channel(binstar_cli, owner, metadata, channel='main'):
    """
    Add a(n already existing) distribution on binstar to another channel.

    Note - the addition is done based on name and version - no build strings etc.
    so if you have a foo-0.1-np18 and foo-0.1-np19 *both* will be added to the channel.

    """
    package_fname = '{}/{}.tar.bz2'.format(subdir, metadata.dist())
    binstar_cli.add_channel(channel, owner, metadata.name(), metadata.version())#filename=package_fname)


def copy_distribution_to_owner(binstar_cli, source_owner, dest_owner,
                               metadata, channel='main'):
    """
    Copy an already existing distribution from one owner to another on
    anaconda.
    """
    package_fname = '{}/{}.tar.bz2'.format(subdir, metadata.dist())
    binstar_cli.copy(source_owner, metadata.name(), metadata.version(),
                     basename=package_fname, to_owner=dest_owner,
                     to_channel=channel)



import os
import unittest

try:
    import conda_build.api
except ImportError:
    import conda_build.config

from conda_build_all.builder import list_metas
from conda_build_all.tests.integration.test_builder import RecipeCreatingUnit


class Test_list_metas(RecipeCreatingUnit):
    def setUp(self):
        super(Test_list_metas, self).setUp()
        m1 = self.write_meta('m1', """
            package:
                name: m1
            """)
        m2 = self.write_meta('.', """
            package:
                name: m2
            """)
        m3 = self.write_meta('d1/d2/d3/meta3', """
            package:
                name: m3
            """)
        m4 = self.write_meta('da1/da2/da3/meta4', """
            package:
                name: m4
            """)

    def test_depth_0(self):
        metas = list_metas(self.recipes_root_dir, max_depth=0)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m1', 'm2', 'm3', 'm4'])

    def test_depth_m1(self):
        metas = list_metas(self.recipes_root_dir, max_depth=-1)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m1', 'm2', 'm3', 'm4'])

    def test_depth_1(self):
        metas = list_metas(self.recipes_root_dir, max_depth=1)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m2'])

    def test_depth_2(self):
        metas = list_metas(self.recipes_root_dir, max_depth=2)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m1', 'm2'])

    def test_default_depth(self):
        metas = list_metas(self.recipes_root_dir)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m1', 'm2', 'm3', 'm4'])

    def test_follow_symlink(self):
        link_dir = self.tmp_dir(prefix='recipes_through_links')
        os.symlink(os.path.join(self.recipes_root_dir, 'd1'),
                os.path.join(link_dir, 'd1'))
        os.symlink(os.path.join(self.recipes_root_dir, 'm1'),
                os.path.join(link_dir, 'm1'))
        metas = list_metas(link_dir)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m1', 'm3'])


class Test_sort_dependency_order(RecipeCreatingUnit):
    def setUp(self):
        super(Test_sort_dependency_order, self).setUp()
        a = self.write_meta('a', """
            package:
                name: a
            requirements:
                build:
                    - c
            """)

        b = self.write_meta('b', """
            package:
                name: b
            requirements:
                run:
                    - a  # [False]
            """)
        c = self.write_meta('c', """
            package:
                name: c
            """)

    def test_order_dependent_selector(self):
        # If we listen to the selectors, we would get a different build order.
        # As a result of https://github.com/SciTools/conda-build-all/issues/30
        # we know that we either have to resolve all dependencies up-front,
        # or simply ignore all selectors when dealing with sort order (but
        # emphatically not when building!).
        if hasattr(conda_build, 'api'):
            config = conda_build.api.Config()
        else:
            config = conda_build.config.config

        metas = list_metas(self.recipes_root_dir)
        from conda_build_all.builder import sort_dependency_order
        names = [m.name() for m in sort_dependency_order(metas, config)]
        self.assertEqual(names, ['c', 'a', 'b'])

 
if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from conda import __version__ as CONDA_VERSION

CONDA_VERSION_MAJOR_MINOR = tuple(int(x) for x in CONDA_VERSION.split('.')[:2])

if (4, 3) <= CONDA_VERSION_MAJOR_MINOR < (4, 4):
    from conda.lock import Locked
    from conda.exports import get_index
    from conda.exports import subdir
    from conda.exports import MatchSpec
    from conda.exports import Unsatisfiable
    from conda.exports import NoPackagesFound
    from conda.exports import Resolve
    from conda.models.dist import Dist as _Dist

    def get_key(dist_or_filename):
        return dist_or_filename

    def copy_index(index):
        return {_Dist(key): index[key] for key in index.keys()}

    def ensure_dist_or_dict(fn):
        return _Dist.from_string(fn)

    from conda.console import setup_verbose_handlers
    setup_verbose_handlers()
    from conda.gateways.logging import initialize_logging
    initialize_logging()

elif (4, 2) <= CONDA_VERSION_MAJOR_MINOR < (4, 3):
    from conda.lock import Locked
    from conda.exports import get_index
    from conda.exports import subdir
    from conda.exports import MatchSpec
    from conda.exports import Unsatisfiable
    from conda.exports import NoPackagesFound
    from conda.exports import Resolve

    def get_key(dist_or_filename):
        return dist_or_filename.fn

    def copy_index(index):
        index = index.copy()
        return index

    def ensure_dist_or_dict(fn):
        return fn

    # We need to import conda.fetch and conda.resolve to trigger the
    # creation of the loggers.
    import conda.fetch
    import conda.resolve

else:
    raise NotImplementedError("CONDA_VERSION: %s  CONDA_VERSION_MAJOR_MINOR: %s"
                              % (CONDA_VERSION, str(CONDA_VERSION_MAJOR_MINOR)))


subdir = subdir
Locked = Locked
Resolve, get_index = Resolve, get_index
MatchSpec = MatchSpec
Unsatisfiable, NoPackagesFound = Unsatisfiable, NoPackagesFound
